(module run mzscheme
  (require (lib "cmdline.ss")
           (lib "file.ss")
           (lib "web-server.ss" "web-server")
           (lib "responders.ss" "web-server" "configuration")
           (lib "namespace.ss" "web-server" "configuration")
           (lib "mime-types.ss" "web-server" "private")
           (lib "cache-table.ss" "web-server" "private")
           (prefix flapjax: "flapjax-dispatcher.ss")
           (prefix log: (lib "dispatch-log.ss" "web-server" "dispatchers"))
           (prefix fsmap: (lib "filesystem-map.ss" "web-server" "dispatchers"))
           (prefix timeout: (lib "dispatch-timeout.ss" "web-server" "dispatchers"))
           (prefix files: (lib "dispatch-files.ss" "web-server" "dispatchers"))
           (prefix filter: (lib "dispatch-filter.ss" "web-server" "dispatchers"))
           (prefix lift: (lib "dispatch-lift.ss" "web-server" "dispatchers"))
           (prefix sequencer: (lib "dispatch-sequencer.ss" "web-server" "dispatchers"))
           (prefix servlets: (lib "dispatch-servlets.ss" "web-server" "dispatchers"))
           (prefix stat: (lib "dispatch-stat.ss" "web-server" "dispatchers")))
  
  (define server-root-path (make-parameter (string->path "web-root")))
  
  (define port (make-parameter 8080))
  
  (parse-command-line 
   "run" (current-command-line-arguments)
   `((once-each
      [("-p" "--port") 
       ,(lambda (flag the-port) (port (string->number the-port)))
       (,(format "Specify a different port (default: ~a)" (number->string (port)))
        "number")]
      [("-r" "--root") 
       ,(lambda (flag path) (server-root-path (normalize-path (string->path path))))
       (,(format "Specify a different server root path (default: ~a)" (path->string (server-root-path)))
        "path")]))
   (lambda (flag-accum) (void))
   null)
  
  (define default-host-path (build-path (server-root-path) "conf"))  
  (define file-not-found-file (build-path default-host-path "not-found.html"))
  (define servlet-error-file (build-path default-host-path "servlet-error.html"))
  
  (define url->path
    (fsmap:make-url->path 
     (build-path (server-root-path) "htdocs")))
  
  (define servlet-url->path
    (fsmap:make-url->path 
     (build-path (server-root-path))))
  
  (define gc-thread (stat:make-gc-thread 30))
  
  (define the-cache-table (box (make-cache-table)))
  (define the-namespace
    (make-make-servlet-namespace
     #:to-be-copied-module-specs null))
  
  (serve #:port (port)
         #:dispatch
         (sequencer:make          
          (timeout:make (* 5 60))
          ; displays memory usage statistics in the log:  (stat:make)
          (log:make #:format log:paren-format #:log-path "log")
          (let-values ([(obj-get obj-set) (flapjax:make-obj-table)])
            (sequencer:make
             (filter:make #rx"^/fxobj/get" obj-get)
             (filter:make #rx"^/fxobj/set" obj-set)))
          (let-values ([(list-get list-cons) (flapjax:make-list-table)])
            (sequencer:make
             (filter:make #rx"^/fxlist/get" list-get)
             (filter:make #rx"^/fxlist/cons" list-cons)))
          (let-values ([(obj-get obj-set) 
                        (flapjax:make-per-address-table flapjax:make-obj-table)])
            (sequencer:make
             (filter:make #rx"^/ip/fxobj/get" obj-get)
             (filter:make #rx"^/ip/fxobj/set" obj-set)))
          (let-values ([(list-get list-cons) 
                        (flapjax:make-per-address-table flapjax:make-list-table)])
            (sequencer:make
             (filter:make #rx"^/ip/fxlist/get" list-get)
             (filter:make #rx"^/ip/fxlist/cons" list-cons)))
          (filter:make
           #rx"^/servlets"
           (let-values ([(refresh dispatcher) ; don't add refresh to the dispatch sequence!
                         (servlets:make 
                          the-cache-table
                          #:make-servlet-namespace the-namespace
                          #:url->path (fsmap:make-url->valid-path servlet-url->path)
                          #:responders-servlet-loading (gen-servlet-responder servlet-error-file)
                          #:responders-servlet (gen-servlet-responder servlet-error-file))])
             dispatcher))
          (files:make #:url->path url->path
                      #:path->mime-type (make-path->mime-type (build-path (server-root-path) "mime.types"))
                      #:indices (list "index.html" "index.htm"))
          (lift:make (gen-file-not-found-responder file-not-found-file))))
  
  (do-not-return))
